/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/7
 */

package agentspace.serial;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InvalidClassException;
import java.io.NotSerializableException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OptionalDataException;
import java.io.Serializable;

public class MarshallingUtil {

    /**
     * <p>Marshals an <code>Object</code> to a byte array for storage/serialization.</p>
     *
     * @param obj the serializable object to serialize to bytes
     * @return a byte[] with the converted Serializable
     */
    public static byte[] marshal(Serializable obj) 
    throws InvalidClassException, NotSerializableException, IOException	 {
    	byte[] data = null;
        ByteArrayOutputStream baos = new ByteArrayOutputStream(1024);
        MarshalOutputStream mos = new MarshalOutputStream(baos);
        try { 
        	mos.writeObject(obj);
        }
        finally {
        	if (mos != null) {
        		mos.close();
        		baos.close();
        	}
        }
        data = baos.toByteArray();
        return data;
    }

    /**
     * <p>Unmarshals an <code>Object</code> from an array of bytes.</p>
     *
     * @param data  the serialized object, must not be null
     * @return the deserialized object
     */
    public static Object unmarshal(byte[] data, ClassLoader loader) 
    throws ClassNotFoundException, IOException, OptionalDataException {
        Object obj = null;
        if (data == null) {
            throw new IllegalArgumentException("The byte[] must not be null");
        }
        ByteArrayInputStream bais = new ByteArrayInputStream(data);
        MarshalInputStream mis = new MarshalInputStream(bais, loader);
        try {
        	if (mis == null) {
        		throw new IllegalArgumentException("The InputStream must not be null");
        	}
        	obj = (Object)mis.readObject();
        } 
        finally {
        	if (mis != null) {
        		mis.close();
        		bais.close();
        	}
        }
        return obj;
    }
    
    
    public static byte[] serialize(Serializable obj) {
        byte[] data = null;
        ByteArrayOutputStream baos = new ByteArrayOutputStream(1024);
        try {
            ObjectOutputStream oos = new ObjectOutputStream(baos);
            MarshallingUtil.serialize(obj, oos);
            data = baos.toByteArray();
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
        return data;
    }
    
    public static Object deserialize(byte[] data) {
        Object obj = null;
        try {
            ByteArrayInputStream bais = new ByteArrayInputStream(data);
            ObjectInputStream ois = new ObjectInputStream(bais);
            obj = deserialize(ois);
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
        return data;
    }
    
    /**
     * <p>Serializes an <code>Object</code> to a stream.</p>
     *
     * @param obj  the object to serialize to bytes, may be null
     * @param os  the stream to write to, must not be null
     * @throws IOException if the serialization fails
     */
    public static void serialize(Object obj, ObjectOutputStream oos) throws IOException {
        if (oos == null) {
            throw new IllegalArgumentException("The OutputStream is null");
        }
        try {
            oos.writeObject(obj);
        } 
        finally {
            try {
                if (oos != null) {
                    oos.close();
                }
            } catch (IOException ex) {}
        }
    }
    
    /**
     * <p>Deserializes an <code>Object</code> from the stream.</p>
     *
     * @param ois the serialized object input stream, must not be null
     * @return the deserialized object
     * @throws IOException if the deserialization fails
     * @throws ClassNotFoundException if the serialization fails
     */
    public static Object deserialize(ObjectInputStream oin) 
    throws IOException, ClassNotFoundException { 
        if (oin == null) {
            throw new IllegalArgumentException("The InputStream must not be null");
        }
        try {
            return (Object)oin.readObject();
        } 
        finally {
            try {
                if (oin != null) {
                    oin.close();
                }
            } catch (IOException ex) {}
        }
    }
}